@@ -24,7 +24,7 @@ __email__ = "nesaro@gmail.com"
 
 import logging
 LOG = logging.getLogger(__name__)
-from pydsl.Grammar.Definition import String
+from pydsl.Grammar.Definition import Grammar, String
 
 class Symbol(object):
     def __init__(self, weight):
@@ -57,6 +57,8 @@ class NonTerminalSymbol(Symbol):
 class TerminalSymbol(Symbol):
 
     def __init__(self, gd, weight=None, boundariesrules=None):
+        if not isinstance(gd, Grammar):
+            raise TypeError
         if isinstance(gd, String):
             weight = weight or 99
             boundariesrules = len(gd)
